/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: parse.c,v 1.9.4.1 2002/03/06 17:06:40 lloydy Exp $
 *
 */
/* command parsing engine */
#include "parse.h"

/* parse.c - Chunky Monkey IRCD Parsing Code
 * Copyright Chris Plant (c) 2000-2001
 * Licensed Under The GNU GPL Version 2, or any later version
 *  
 * This Code parses every command to pass into the ircd
 * :origin COMMAND param1 ... param15 :Text
 * is parsed into 
 * struct parsed_cmd {
 *	char *origin;
 *	char *command;
 *	char *params[15];
 *	char *text;
 * };
 * which is malloc'ed and free'ed so that only one *should* exist at any
 * one time */ 

static Cmd last_alloc=NULL;

/* 
 * Takes input, and create a Cmd for other routines to use
 */
Cmd parse_command(char *command)
{
	Cmd cmd=NULL;
	int i,j,last,str_len,start=0;
	char * text=NULL;
	char * params=NULL;
	char *cmd_start=NULL;
	

	/* Allocate memory for parsed cmd */
	if(last_alloc==NULL) {
		cmd=ircd_malloc(sizeof(struct _parsed_cmd));
		last_alloc=cmd;
	} else {
		printf("Memory Leak with commands\n");
		exit(1);
	}

	/* Strip \n\r off the end */
	str_len=strlen(command);
	for(i=str_len;command[i]=='\n' || command[i]=='\r' ||command[i]=='\0';i--) {
		command[i]='\0';
	}
	
	str_len = strlen(command);
	
	/* Get the origin */
	if(command[0]==':') {
		/* We have an origin */
		/* Find First Space */
		for(i=1;i<str_len && start == 0;i++) {
			if(command[i]==' ') 
				start = i;
		}
		if(start) {
			cmd->origin=string_alloc(start+1);
			strncpy(cmd->origin,&(command[1]),start-1);
		}

	} else {
		/* We don't */
		cmd->origin=NULL;
	}

	/* Locate start of text after : */
	last=-1;
	for(i=str_len;i>0 ;i--) {
		if(command[i]==':') {
			cmd->had_colon=1;
			last=i;
			text=&(command[i+1]);
		} 			
	}	
	if(last!=0) 
		command[last]='\0';

	/* Copy text into cmd->text */
	if(text) {
		str_len=strlen(text);
		/* If we get :\0, this will stop it adding a text ptr */
		if(str_len) {
			cmd->text=string_alloc(str_len);
			strncpy(cmd->text,text,str_len);
			cmd->text[str_len]='\0';
		}
	}
	
	
	/* Grab Command */
	str_len=strlen(command);
	if(command[0]==':') {
		/* Its prefixed */
		/* Find actual start */
		for(i=0;i<str_len && cmd_start==NULL;i++) {
			if(command[i]==' ') 
				cmd_start=&(command[i+1]);
		}
		 
	} else {
		cmd_start=command;	
	}

	if(!cmd_start) {
		return NULL;
	}
	
	str_len=strlen(cmd_start);
	for(i=0;params==NULL && i<str_len;i++) {
		if(cmd_start[i]==' ') {
			cmd_start[i]='\0';
			params=&(cmd_start[i+1]);
		}
	}

	/* Alloc space */
	cmd->command=string_alloc(i);
	strcpy(cmd->command,cmd_start);

	
	/* Process params list */
	if(params!=NULL) {
		str_len=strlen(params);
		last=0;
		j=0;
		for(i=0;i<str_len+1 && j < 15;i++) {
			if(params[i]==' ' || params[i]==',' || params[i]=='\0') {
				if(last!=i) {
					params[i]='\0';
					if( (i-last) < 101) {
						cmd->params[j]=string_alloc(i-last);
						strcpy(cmd->params[j++],&(params[last]));
					} else {
						cmd->params[j++]=NULL;
					}
					last=i+1;
				}
			}
		
		}
	}

	if(!cmd->params[0]) {
		cmd->params[0]=cmd->text;
	}

	/* Temp hack */
	/* Will leave if it works */
	return cmd;	
}

/* 
 * function called to clean up command structs 
 */
int cmd_free(Cmd freeme)
{
	int i;

	if(last_alloc!=freeme) {
		printf("Something wrong with command allocation\n");
		exit(1);
	} else {
		last_alloc=NULL;
	}	
	
	if(freeme==NULL)
		return 1;
	
	/* Free text and command storage */	
	if(freeme->text!=NULL) {
		if(freeme->text==freeme->params[0]) {
			freeme->params[0]=NULL;
		}
		free(freeme->text);
		freeme->text=NULL;
	}
	if(freeme->command!=NULL)
		free(freeme->command);
	if(freeme->origin)
		free(freeme->origin);

	/* Loop though parameter array */
	/* free each if needed */
	if(freeme->params[0]!=NULL)
		for(i=0;freeme->params[i]!=NULL && i < 15;i++) 
			free(freeme->params[i]);
	
	/* Free main structure */
	free(freeme);

	return 1;
}

/* Check we have the requested number of params */
int check_params(Cmd p_cmd,int num_params)
{
	int i;

	for(i=0;i<num_params;i++) {
		if(p_cmd->params[i]==NULL) 
			return 0;
	}

	return 1;
}
